@@ -1,6 +1,7 @@
 /*
 Copyright 2008, 2009 Will Stephenson <wstephenson@kde.org>
 Copyright 2008, 2009 Sebastian Kügler <sebas@kde.org>
+Copyright 2011-2012 Lamarque V. Souza <lamarque@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -101,8 +102,7 @@ NetworkManagerApplet::NetworkManagerApplet(QObject * parent, const QVariantList
     m_meterFgSvg = new Plasma::FrameSvg(this);
     m_meterFgSvg->setImagePath("widgets/bar_meter_horizontal");
     m_meterFgSvg->setElementPrefix("bar-active");
-    setStatus(Plasma::ActiveStatus);
-    m_interfaces = Solid::Control::NetworkManagerNm09::networkInterfaces();
+    updateInterfaceList();
     if (!m_interfaces.isEmpty()) {
         qSort(m_interfaces.begin(), m_interfaces.end(), networkInterfaceLessThan);
         setActiveInterface(m_interfaces.first());
@@ -116,7 +116,7 @@ NetworkManagerApplet::NetworkManagerApplet(QObject * parent, const QVariantList
     kded.call(QLatin1String("loadModule"), QLatin1String("networkmanagement"));
 
     m_activatables = new RemoteActivatableList(this);
-    connect(m_activatables, SIGNAL(activatableAdded(RemoteActivatable*, int)), this, SLOT(activatableAdded(RemoteActivatable*)));
+    connect(m_activatables, SIGNAL(activatableAdded(RemoteActivatable*,int)), this, SLOT(activatableAdded(RemoteActivatable*)));
     connect(m_activatables, SIGNAL(activatableRemoved(RemoteActivatable*)), this, SLOT(activatableRemoved(RemoteActivatable*)));
     connect(m_activatables, SIGNAL(disappeared()), this, SLOT(activatablesDisappeared()));
 }
@@ -221,7 +221,10 @@ QString NetworkManagerApplet::svgElement(Solid::Control::NetworkInterfaceNm09 *i
                     case Solid::Control::ModemInterface::Hspa:
                         return QString("network-mobile-%1-hspa").arg(strength);
                     default:
-                        return QString("network-mobile-%1-none").arg(strength);
+                        // FIXME: this icon exists but it is invisible, we need
+                        // to create a better one.
+                        //return QString("network-mobile-%1-none").arg(strength);
+                        return QString();
                 }
             }
         }
@@ -241,7 +244,7 @@ void NetworkManagerApplet::setupInterfaceSignals()
 {
     foreach (Solid::Control::NetworkInterfaceNm09* interface, m_interfaces) {
         // be aware of state changes
-        QObject::disconnect(interface, SIGNAL(connectionStateChanged(int, int, int)), this, SLOT(interfaceConnectionStateChanged()));
+        QObject::disconnect(interface, SIGNAL(connectionStateChanged(int,int,int)), this, SLOT(interfaceConnectionStateChanged()));
         QObject::disconnect(interface, SIGNAL(connectionStateChanged(int)), this, SLOT(interfaceConnectionStateChanged()));
         QObject::disconnect(interface, SIGNAL(linkUpChanged(bool)));
 
@@ -260,7 +263,7 @@ void NetworkManagerApplet::setupInterfaceSignals()
         } else if (interface->type() == Solid::Control::NetworkInterfaceNm09::Wifi) {
             Solid::Control::WirelessNetworkInterfaceNm09* wirelessiface =
                             static_cast<Solid::Control::WirelessNetworkInterfaceNm09*>(interface);
-            connect(wirelessiface, SIGNAL(activeAccessPointChanged(const QString&)), SLOT(setupAccessPointSignals(const QString&)));
+            connect(wirelessiface, SIGNAL(activeAccessPointChanged(QString)), SLOT(setupAccessPointSignals(QString)));
             QMetaObject::invokeMethod(wirelessiface, "activeAccessPointChanged",
                                       Q_ARG(QString, wirelessiface->activeAccessPoint()));
         } else if (interface->type() == Solid::Control::NetworkInterfaceNm09::Modem) {
@@ -301,10 +304,10 @@ void NetworkManagerApplet::init()
     }
 
     //kDebug();
-    QObject::connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceAdded(const QString&)),
-            this, SLOT(networkInterfaceAdded(const QString&)));
-    QObject::connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceRemoved(const QString&)),
-            this, SLOT(networkInterfaceRemoved(const QString&)));
+    QObject::connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceAdded(QString)),
+            this, SLOT(networkInterfaceAdded(QString)));
+    QObject::connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(networkInterfaceRemoved(QString)),
+            this, SLOT(networkInterfaceRemoved(QString)));
 
     QObject::connect(Solid::Control::NetworkManagerNm09::notifier(), SIGNAL(statusChanged(Solid::Networking::Status)),
                      this, SLOT(managerStatusChanged(Solid::Networking::Status)));
@@ -316,19 +319,6 @@ void NetworkManagerApplet::init()
         connect(m_popup, SIGNAL(configNeedsSaving()), this, SIGNAL(configNeedsSaving()));
     }
 
-    // m_activatables->init() must be called after SLOT(activatableAdded(RemoteActivatable*)) has been connected and
-    // NMPopup has been allocated.
-    m_activatables->init();
-    interfaceConnectionStateChanged();
-
-    // to force InterfaceItems to update their hasDefaultRoute state.
-    if (m_activeInterface) {
-        QMetaObject::invokeMethod(m_activeInterface, "connectionStateChanged",
-                                  Q_ARG(int, m_activeInterface->connectionState()),
-                                  Q_ARG(int, Solid::Control::NetworkInterfaceNm09::UnknownState),
-                                  Q_ARG(int, Solid::Control::NetworkInterfaceNm09::NoReason));
-    }
-
     QDBusConnection dbus = QDBusConnection::sessionBus();
     dbus.connect("org.kde.kded", "/org/kde/networkmanagement", "org.kde.networkmanagement", "ModuleReady", this, SLOT(finishInitialization()));
 
@@ -344,6 +334,21 @@ void NetworkManagerApplet::finishInitialization()
                                      QLatin1String("org.kde.networkmanagement"), QDBusConnection::sessionBus());
 
     networkmanagement.call(QLatin1String("FinishInitialization"));
+
+    // m_activatables->init() must be called after SLOT(activatableAdded(RemoteActivatable*,int)) has been connected and
+    // NMPopup has been allocated.
+    m_activatables->init();
+
+    // this needs m_activables initialized so that it can get the connection name to add to the InterfaceItem.
+    interfaceConnectionStateChanged();
+
+    // to force InterfaceItems to update their hasDefaultRoute state.
+    if (m_activeInterface) {
+        QMetaObject::invokeMethod(m_activeInterface, "connectionStateChanged",
+                                  Q_ARG(int, m_activeInterface->connectionState()),
+                                  Q_ARG(int, Solid::Control::NetworkInterfaceNm09::UnknownState),
+                                  Q_ARG(int, Solid::Control::NetworkInterfaceNm09::NoReason));
+    }
 }
 
 QGraphicsWidget* NetworkManagerApplet::graphicsWidget()
@@ -364,6 +369,8 @@ void NetworkManagerApplet::createConfigurationInterface(KConfigDialog *parent)
     parent->addPage(m_kcmNMTray, m_kcmNMTray->moduleInfo().moduleName(),
                     m_kcmNMTray->moduleInfo().icon());
 
+    parent->resize(800, 600);
+
     connect(parent, SIGNAL(applyClicked()), this, SLOT(saveConfiguration()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(saveConfiguration()));
 }
@@ -392,8 +399,14 @@ void NetworkManagerApplet::constraintsEvent(Plasma::Constraints constraints)
 
 void NetworkManagerApplet::updatePixmap()
 {
+    QString iconName = UiUtils::iconName(m_activeSystrayInterface);
+    if (iconName == m_currentPixmapIconName) {
+        return;
+    }
+
     int s = UiUtils::iconSize(contentsRect().size());
-    m_pixmap = KIcon(UiUtils::iconName(m_activeSystrayInterface)).pixmap(s, s);
+    m_currentPixmapIconName = iconName;
+    m_pixmap = KIcon(m_currentPixmapIconName).pixmap(s, s);
     update();
 }
 
@@ -405,9 +418,17 @@ void NetworkManagerApplet::paintInterface(QPainter * p, const QStyleOptionGraphi
     if (!m_panelContainment) {
         /* To make applet's size matches the popup's size. The applet is the tray icon, which is 16x16 pixels size by default.*/
         adjustSize();
+    }
+
+    QString el = svgElement(m_activeSystrayInterface);
+
+    if ((el == m_currentSvgElement || (el.isEmpty() && m_currentSvgElement == m_currentPixmapIconName)) &&
+        !needToUpdateOverlay()) {
+        // no need to update systray icon
         return;
     }
 
+
     /* I am using setPopupIcon at the end of this method to make the usual system tray icon's hover and click
      * effects work. However, setPopupIcon creates a Plasma::IconWidget object that draws itself over
      * contentsRect and, consequentely, over the overlays created by paintStatusOverlay and paintNeedAuthOverlay.
@@ -424,11 +445,12 @@ void NetworkManagerApplet::paintInterface(QPainter * p, const QStyleOptionGraphi
     QPainter painter;
     painter.begin(&newIcon);
 
-    QString el = svgElement(m_activeSystrayInterface);
     if (el.isEmpty()) {
+        m_currentSvgElement = m_currentPixmapIconName;
         painter.drawPixmap(QPoint(0,0), m_pixmap);
     } else {
-        if (el.startsWith("network-mobile")) {
+        m_currentSvgElement = el;
+        if (el.startsWith(QLatin1String("network-mobile"))) {
             m_svgMobile->paint(&painter, rect, el);
         } else {
             m_svg->paint(&painter, rect, el);
@@ -441,17 +463,42 @@ void NetworkManagerApplet::paintInterface(QPainter * p, const QStyleOptionGraphi
     setPopupIcon(newIcon);
 }
 
+bool NetworkManagerApplet::needToUpdateOverlay()
+{
+    SystrayOverlayOptions temp;
+
+    if (m_activeSystrayInterface && m_activeSystrayInterface->connectionState() == Solid::Control::NetworkInterfaceNm09::NeedAuth) {
+        temp |= NetworkManagerApplet::NeedAuth;
+    }
+
+    if (m_totalActiveVpnConnections > 0) {
+        temp |= NetworkManagerApplet::Locked;
+    }
+
+    qreal opacity = m_overlayTimeline.currentValue();
+    if (!qFuzzyCompare(opacity, 1) && !m_previousStatusOverlay.isNull()) {
+        temp |= NetworkManagerApplet::PreviousOverlay;
+    }
+
+    if (!m_statusOverlay.isNull()) {
+        temp |= NetworkManagerApplet::StatusOverlay;
+    }
+
+    return (temp != m_systrayOverlayOption);
+}
+
 inline void NetworkManagerApplet::paintNeedAuthOverlay(QPainter *p, QRect &rect)
 {
     // Needs authentication, show this in the panel
     if (!m_activeSystrayInterface) {
+        m_systrayOverlayOption &= ~NetworkManagerApplet::NeedAuth;
         kDebug() << "No active interface";
         return;
     }
     /*
     kDebug() << "Painting overlay ...>" << m_activeSystrayInterface->connectionState();
     */
-    if (m_activeSystrayInterface && m_activeSystrayInterface->connectionState() == Solid::Control::NetworkInterfaceNm09::NeedAuth) {
+    if (m_activeSystrayInterface->connectionState() == Solid::Control::NetworkInterfaceNm09::NeedAuth) {
         //kDebug() << "Needing auth ...>";
         int iconSize = (int)2*(rect.width()/3);
 
@@ -461,6 +508,9 @@ inline void NetworkManagerApplet::paintNeedAuthOverlay(QPainter *p, QRect &rect)
                             rect.bottom() - iconSize);
 
         p->drawPixmap(pos, icon);
+        m_systrayOverlayOption |= NetworkManagerApplet::NeedAuth;
+    } else {
+        m_systrayOverlayOption &= ~NetworkManagerApplet::NeedAuth;
     }
 }
 
@@ -470,6 +520,9 @@ inline void NetworkManagerApplet::paintStatusOverlay(QPainter *p, QRect &rect)
         int iconSize = (int)2*(rect.width()/3);
         QPixmap pix = KIcon("object-locked").pixmap(iconSize);
         p->drawPixmap(rect.right() - pix.width(), rect.bottom() - pix.height(), pix);
+        m_systrayOverlayOption |= NetworkManagerApplet::Locked;
+    } else {
+        m_systrayOverlayOption &= ~NetworkManagerApplet::Locked;
     }
 
     int oldOpacity = p->opacity();
@@ -477,10 +530,17 @@ inline void NetworkManagerApplet::paintStatusOverlay(QPainter *p, QRect &rect)
     if (!qFuzzyCompare(opacity, 1) && !m_previousStatusOverlay.isNull()) {
         p->setOpacity(1 - opacity);
         p->drawPixmap(rect.left(), rect.bottom() - m_previousStatusOverlay.height(), m_previousStatusOverlay);
+        m_systrayOverlayOption |= NetworkManagerApplet::PreviousOverlay;
+    } else {
+        m_systrayOverlayOption &= ~NetworkManagerApplet::PreviousOverlay;
     }
+
     if (!m_statusOverlay.isNull()) {
         p->setOpacity(opacity);
         p->drawPixmap(rect.left(), rect.bottom() - m_statusOverlay.height(), m_statusOverlay);
+        m_systrayOverlayOption |= NetworkManagerApplet::StatusOverlay;
+    } else {
+        m_systrayOverlayOption &= ~NetworkManagerApplet::StatusOverlay;
     }
     p->setOpacity(oldOpacity);
 }
@@ -495,7 +555,7 @@ void NetworkManagerApplet::networkInterfaceAdded(const QString & uni)
 {
     Q_UNUSED(uni);
     // update the tray icon
-    m_interfaces = Solid::Control::NetworkManagerNm09::networkInterfaces();
+    updateInterfaceList();
 
     if (!m_activeInterface) {
         if (m_interfaces.isEmpty()) {
@@ -513,7 +573,7 @@ void NetworkManagerApplet::networkInterfaceAdded(const QString & uni)
 void NetworkManagerApplet::networkInterfaceRemoved(const QString & uni)
 {
     // update the tray icon
-    m_interfaces = Solid::Control::NetworkManagerNm09::networkInterfaces();
+    updateInterfaceList();
 
     if (uni == m_lastActiveInterfaceUni) {
         if (m_interfaces.isEmpty()) {
@@ -625,7 +685,8 @@ void NetworkManagerApplet::toolTipAboutToShow()
         QString icon = "networkmanager";
         QStringList lines;
         foreach (Solid::Control::NetworkInterfaceNm09 *iface, interfaces) {
-            if (iface->connectionState() != Solid::Control::NetworkInterfaceNm09::Unavailable) {
+            if (iface->connectionState() != Solid::Control::NetworkInterfaceNm09::Unavailable &&
+                iface->connectionState() != Solid::Control::NetworkInterfaceNm09::Unmanaged) {
                 if (!lines.isEmpty()) {
                     lines << QString();
                 }
@@ -876,7 +937,7 @@ void NetworkManagerApplet::userWirelessEnabledChanged(bool enabled)
 void NetworkManagerApplet::managerStatusChanged(Solid::Networking::Status status)
 {
     //kDebug() << "managerstatuschanged";
-    m_interfaces = Solid::Control::NetworkManagerNm09::networkInterfaces();
+    updateInterfaceList();
     if (status == Solid::Networking::Unknown) {
         setActiveInterface(0);
         setActiveSystrayInterface(0);
@@ -953,9 +1014,12 @@ void NetworkManagerApplet::activatableAdded(RemoteActivatable *activatable)
 {
     RemoteInterfaceConnection *ic = qobject_cast<RemoteInterfaceConnection*>(activatable);
     if (activatable->activatableType() == Knm::Activatable::VpnInterfaceConnection) {
-        connect(ic, SIGNAL(activationStateChanged(Knm::InterfaceConnection::ActivationState, Knm::InterfaceConnection::ActivationState)),
-                this, SLOT(vpnActivationStateChanged(Knm::InterfaceConnection::ActivationState, Knm::InterfaceConnection::ActivationState)));
-        QMetaObject::invokeMethod(ic, "activationStateChanged", Q_ARG(Knm::InterfaceConnection::ActivationState, ic->oldActivationState()), Q_ARG(Knm::InterfaceConnection::ActivationState, ic->activationState()));
+        connect(ic, SIGNAL(activationStateChanged(Knm::InterfaceConnection::ActivationState,Knm::InterfaceConnection::ActivationState)),
+                this, SLOT(vpnActivationStateChanged(Knm::InterfaceConnection::ActivationState,Knm::InterfaceConnection::ActivationState)));
+	Knm::InterfaceConnection::ActivationState state = ic->activationState();
+	if (state != Knm::InterfaceConnection::Unknown) {
+            QMetaObject::invokeMethod(ic, "activationStateChanged", Q_ARG(Knm::InterfaceConnection::ActivationState, ic->oldActivationState()), Q_ARG(Knm::InterfaceConnection::ActivationState, state));
+	}
     } else if (ic) {
         connect(ic, SIGNAL(hasDefaultRouteChanged(bool)), SLOT(updateActiveInterface(bool)));
         QMetaObject::invokeMethod(ic, "hasDefaultRouteChanged", Q_ARG(bool, ic->hasDefaultRoute()));
@@ -978,8 +1042,9 @@ void NetworkManagerApplet::vpnActivationStateChanged(Knm::InterfaceConnection::A
             break;
         case Knm::InterfaceConnection::Unknown:
             m_activeVpnConnections.remove(id);
-            if (oldState == Knm::InterfaceConnection::Activated && m_totalActiveVpnConnections > 0)
+            if (oldState >= Knm::InterfaceConnection::Activated && m_totalActiveVpnConnections > 0) {
                 m_totalActiveVpnConnections--;
+            }
             break;
     }
     //kDebug() << newState << m_totalActiveVpnConnections;
@@ -992,7 +1057,7 @@ void NetworkManagerApplet::updateActiveInterface(bool hasDefaultRoute)
     if (hasDefaultRoute) {
         // TODO: add support for VpnRemoteInterfaceConnection's, which have "any" as ic->deviceUni().
         setActiveInterface(Solid::Control::NetworkManagerNm09::findNetworkInterface(ic->deviceUni()));
-        connect(m_activeInterface, SIGNAL(destroyed(QObject *)), SLOT(_k_destroyed(QObject *)));
+        connect(m_activeInterface, SIGNAL(destroyed(QObject*)), SLOT(_k_destroyed(QObject*)));
         resetActiveSystrayInterface();
     }
 }
@@ -1054,4 +1119,10 @@ void NetworkManagerApplet::setActiveSystrayInterface(Solid::Control::NetworkInte
     }
 }
 
+void NetworkManagerApplet::updateInterfaceList()
+{
+    m_interfaces = Solid::Control::NetworkManagerNm09::networkInterfaces();
+    setStatus(m_interfaces.isEmpty() ? Plasma::PassiveStatus : Plasma::ActiveStatus);
+}
+
 #include "networkmanager.moc"
